@@ -19,6 +19,7 @@
 
 #include <gtk/gtk.h>
 
+#include <gpe/question.h>
 #include <gpe/pixmaps.h>
 #include <gpe/picturebutton.h>
 #include <gpe/gtkdatecombo.h>
@@ -26,6 +27,7 @@
 #include <gpe/gtksimplemenu.h>
 #include <gpe/event-db.h>
 #include <gpe/spacing.h>
+#include <gpe/schedule.h>
 
 #include "globals.h"
 #include "event-ui.h"
@@ -73,6 +75,9 @@ struct edit_state
 
 static GtkWidget *cached_window;
 
+/* minute, hour, day, week */
+static const int alarm_multipliers[] = { 60, 60*60, 24*60*60, 7*24*60*60 };
+ 
 static void
 destroy_user_data (gpointer p)
 {
@@ -169,72 +174,22 @@ recalculate_sensitivities (GtkWidget *widget,
 }
 
 static void
-schedule_alarm(event_t ev, time_t start)
+unschedule_alarm (event_t ev)
 {
-  time_t alarm_t;
-  event_details_t ev_d;
-  char filename[256];
-  FILE *f;
-
-  alarm_t = start-60*ev->alarm;
-  ev_d = event_db_get_details (ev);
-
-  sprintf(filename, "/var/spool/at/%ld.%ld.%ld", (long)alarm_t, ev->uid,
-                  (long)(getuid()));
-
-  if ((f=fopen(filename, "w")))
-  {
-    fprintf(f, "#!/bin/sh\n");
-    fprintf(f, "export DISPLAY=:0.0\n");
-    fprintf(f, "/usr/bin/gpe-announce '%s'\n", ev_d->summary);
-    fprintf(f, "/usr/bin/gpe-calendar -s %ld -e %ld &\n", (long)alarm_t, ev->uid);
-    fprintf(f, "/bin/rm $0\n");
-
-    fclose(f);
-
-    chmod (filename, S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH);
-
-    f=NULL;
-    f=fopen("/var/spool/at/trigger","w");
-    if (f != NULL)
-    {
-      fprintf(f,"\n");
-      fclose(f);
-    }
-  }
-}
-
-static void
-unschedule_alarm(event_t ev)
-{
-  time_t alarm_t;
-  char filename[256], command[256];
-  FILE *f;
-
-  alarm_t = ev->start-60*ev->alarm;
-
-  sprintf(filename, "/var/spool/at/%ld.%ld.%ld", (long)alarm_t, ev->uid,
-                  (long)(getuid()));
-  sprintf(command, "rm %s", filename);
-  system(command);
-
-  f=NULL;
-  f=fopen("/var/spool/at/trigger","w");
-  if (f != NULL)
-  {
-    fprintf(f,"\n");
-    fclose(f);
-  }
+  event_t ev_real = (event_t)ev->cloned_ev;
 
+  if (!schedule_cancel_alarm (ev_real->uid, ev->start)) 
+    printf("ack!\n");
 }
 
 void
-schedule_next(guint skip, guint uid)
+schedule_next (guint skip, guint uid)
 {
-  GSList *events=NULL, *iter;
+  GSList *events = NULL, *iter;
   struct tm tm;
-  time_t end, start=time(NULL);
-
+  time_t end, start = time(NULL);
+  event_details_t ev_d;
+  
   localtime_r (&start, &tm);
   tm.tm_year++;
   end=mktime (&tm);
@@ -245,15 +200,25 @@ schedule_next(guint skip, guint uid)
     {
       event_t ev_real, ev = iter->data;
 
-      if (ev->flags & FLAG_ALARM) {
-        ev_real = event_db_find_by_uid (ev->uid);
-        if (((int)(ev->start) != (int)skip) && (uid!=ev->uid))
-          {
-            schedule_alarm (ev_real, ev->start);
-            break;
-          }
-      }
+      if (ev->flags & FLAG_ALARM) 
+	{
+	  ev_real = (event_t)ev->cloned_ev;
+	  ev_d = event_db_get_details (ev_real);
+	  if (((int)(ev->start) != (int)skip) && (uid!=ev_real->uid))
+	    {
+	      gchar *action;
+
+	      action = g_strdup_printf ("/usr/bin/gpe-announce '%s'\n/usr/bin/gpe-calendar -s %ld -e %ld &\n ",
+					ev_d->summary, (long)ev->start, ev_real->uid);
+	      if (!schedule_set_alarm (ev->uid, ev->start, action)) 
+		printf("ack!\n");
+
+	      g_free (action);
+	      break;
+	    }
+	}
     }
+    event_db_list_destroy (events);
 }
 
 static gboolean
@@ -270,19 +235,49 @@ edit_finished (GtkWidget *d)
 }
 
 static void
-click_delete (GtkWidget *widget, event_t ev)
+click_delete (GtkWidget *widget, GtkWidget *dw)
 {
   GtkWidget *d = gtk_widget_get_toplevel (widget);
-
-  if (ev->flags & FLAG_ALARM)
+  struct edit_state *s = g_object_get_data (G_OBJECT (d), "edit_state");
+  event_t ev = s->ev, ev_real;
+  event_details_t ev_d;
+  recur_t r;
+  
+  ev_real = (event_t)ev->cloned_ev;
+  ev_d = event_db_get_details (ev_real);
+     
+  if (ev_real->recur)
     {
-      unschedule_alarm (ev);
-      schedule_next(0,0);
+      if (gpe_question_ask_yn (_("Delete all recurring entries?\n(If no, delete this instance only)")))
+	{			  
+	  if (ev_real->flags & FLAG_ALARM)
+	    {
+	      unschedule_alarm (ev);
+	      schedule_next (0,0);
+	    }
+	  
+	  event_db_remove (ev_real);
+	}
+      else 
+	{
+	  r = event_db_get_recurrence (ev_real);
+	  r->exceptions = g_slist_append (r->exceptions, (void *)ev->start);
+	  event_db_flush (ev_real);
+	}
     }
-
-  event_db_remove (ev);
+  else
+    {			  
+      if (ev_real->flags & FLAG_ALARM)
+	{
+	  unschedule_alarm (ev);
+	  schedule_next (0,0);
+	}
+      
+      event_db_remove (ev_real);
+  }
+  
   update_all_views ();
-
+  event_db_forget_details (ev_real);
   edit_finished (d);
 }
 
@@ -295,13 +290,15 @@ click_ok (GtkWidget *widget, GtkWidget *d)
   struct tm tm_start, tm_end, tm_rend;
   time_t start_t, end_t, rend_t;
   gboolean new_event = FALSE;
-
+  GtkTextIter start, end;
+  GtkTextBuffer *buf;
+    
   if (s->ev)
     {
-      ev = s->ev;
+      ev = (event_t)(s->ev->cloned_ev);
       ev_d = event_db_get_details (ev);
       if (ev->flags & FLAG_ALARM)
-        unschedule_alarm (ev);
+        unschedule_alarm (s->ev);
       ev_d->sequence++;
     }
   else
@@ -394,8 +391,7 @@ click_ok (GtkWidget *widget, GtkWidget *d)
   if (ev_d->description)
     g_free (ev_d->description);
 
-    GtkTextIter start, end;
-    GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (s->description));
+    buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (s->description));
     gtk_text_buffer_get_bounds (buf, &start, &end);
     ev_d->description = gtk_text_buffer_get_text (buf, &start, &end, FALSE);
 
@@ -407,8 +403,12 @@ click_ok (GtkWidget *widget, GtkWidget *d)
 
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (s->alarmbutton)))
     {
+      unsigned int mi;
+ 
+      mi = gtk_option_menu_get_history (GTK_OPTION_MENU (s->alarmoption));
+      
       ev->flags |= FLAG_ALARM;
-      ev->alarm = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (s->alarmspin));
+      ev->alarm = alarm_multipliers[mi] * gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (s->alarmspin));
     }
   else
     ev->flags &= ~FLAG_ALARM;
@@ -417,12 +417,14 @@ click_ok (GtkWidget *widget, GtkWidget *d)
     {
       if (ev->recur)
         g_free (ev->recur);
+      ev->flags |= FLAG_RECUR;
       ev->recur = NULL;
     }
   else
     {
       recur_t r = event_db_get_recurrence (ev);
-
+      ev->flags &= ~FLAG_RECUR;
+      
       if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (s->radiobuttondaily)))
         {
           r->type = RECUR_DAILY;
@@ -622,21 +624,29 @@ build_edit_event_window (void)
   labelalarmpage      = gtk_label_new (_("Alarm"));
 
   vboxevent           = gtk_vbox_new (FALSE, boxspacing);
-  vboxrecur           = gtk_vbox_new (FALSE, boxspacing);
   vboxalarm           = gtk_vbox_new (FALSE, boxspacing);
+  vboxrecur           = gtk_vbox_new (FALSE, boxspacing);
   vboxappointment     = gtk_vbox_new (FALSE, 0);
-  vboxend             = gtk_vbox_new (FALSE, 0);
   vboxreminder        = gtk_vbox_new (FALSE, 0);
-  vboxrepeat          = gtk_vbox_new (FALSE, 0);
   vboxtask            = gtk_vbox_new (FALSE, 0);
+  vboxrepeattop       = gtk_vbox_new (FALSE, 0);
+  vboxrepeat          = gtk_vbox_new (FALSE, 0);
+  vboxend             = gtk_vbox_new (FALSE, 0);
+  vboxtop             = gtk_vbox_new (FALSE, 0);
   vboxtop             = gtk_vbox_new (FALSE, 0);
-
   hboxendafter        = gtk_hbox_new (FALSE, 0);
   hboxtask1           = gtk_hbox_new (FALSE, 0);
   hboxtask2           = gtk_hbox_new (FALSE, 0);
 
   s                   = g_malloc0 (sizeof (struct edit_state));
 
+  /* Scrolled window for event tab */
+  scrolledwindowevent = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindowevent),
+                                         vboxevent);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindowevent),
+                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+
   menutypehbox        = gtk_hbox_new (FALSE, boxspacing);
   menutypelabel       = gtk_label_new (_("Type:"));
   gtk_misc_set_alignment (GTK_MISC (menutypelabel), 0.0, 0.5);
@@ -682,10 +692,10 @@ build_edit_event_window (void)
   starttimelabel      = gtk_label_new (_("at:"));
   endtimelabel        = gtk_label_new (_("at:"));
 
-  s->startdate = gtk_date_combo_new ();
-  s->enddate = gtk_date_combo_new ();
-  s->reminderdate = gtk_date_combo_new ();
-  s->taskdate = gtk_date_combo_new ();
+  s->startdate        = gtk_date_combo_new ();
+  s->enddate          = gtk_date_combo_new ();
+  s->reminderdate     = gtk_date_combo_new ();
+  s->taskdate         = gtk_date_combo_new ();
 
   gtk_date_combo_week_starts_monday (GTK_DATE_COMBO (s->startdate), week_starts_monday);
   gtk_date_combo_week_starts_monday (GTK_DATE_COMBO (s->enddate), week_starts_monday);
@@ -771,12 +781,49 @@ build_edit_event_window (void)
   gtk_box_pack_start (GTK_BOX (vboxevent), menutypehbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vboxevent), summaryhbox, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vboxevent), s->notebooktype, FALSE, FALSE, 0);
-
   gtk_box_pack_start (GTK_BOX (vboxevent), descriptionlabel, FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vboxevent), descriptionhbox, TRUE, TRUE, 0);
-
   gtk_container_set_border_width (GTK_CONTAINER (vboxevent), border);
+
+  /* Alarm page */
+  alarmhbox1          = gtk_hbox_new (FALSE, boxspacing);
+  alarmhbox2          = gtk_hbox_new (FALSE, boxspacing);
+  scrolledwindowalarm = gtk_scrolled_window_new (NULL, NULL);
+  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindowalarm),
+                                         vboxalarm);
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindowalarm),
+                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+  alarmmenu           = gtk_menu_new ();
+  alarmbutton         = gtk_check_button_new_with_label (_("Alarm"));
+  alarmadj            = gtk_adjustment_new (5.0, 1.0, 10000.0, 1.0, 5.0, 5.0);
+  alarmspin           = gtk_spin_button_new (GTK_ADJUSTMENT (alarmadj), 1.0, 0);
+  alarmoption         = gtk_option_menu_new ();
+  alarmlabel          = gtk_label_new (_("before event"));
+
+  gtk_menu_append (GTK_MENU (alarmmenu),
+                   gtk_menu_item_new_with_label (_("minutes")));
+  gtk_menu_append (GTK_MENU (alarmmenu),
+                   gtk_menu_item_new_with_label (_("hours")));
+  gtk_menu_append (GTK_MENU (alarmmenu),
+                   gtk_menu_item_new_with_label (_("days")));
+  gtk_menu_append (GTK_MENU (alarmmenu),
+                   gtk_menu_item_new_with_label (_("weeks")));
+
+  gtk_option_menu_set_menu (GTK_OPTION_MENU (alarmoption), alarmmenu);
+
+  g_signal_connect (G_OBJECT (alarmbutton), "clicked",
+                    G_CALLBACK (recalculate_sensitivities), window);
+
+  gtk_box_pack_start (GTK_BOX (alarmhbox1), alarmbutton, FALSE, FALSE, 0);
+
+  gtk_box_pack_start (GTK_BOX (alarmhbox2), alarmspin, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (alarmhbox2), alarmoption, TRUE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (alarmhbox2), alarmlabel, FALSE, FALSE, 0);
+
+  gtk_box_pack_start (GTK_BOX (vboxalarm), alarmhbox1, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vboxalarm), alarmhbox2, FALSE, FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (vboxalarm), border);
+
   gtk_container_set_border_width (GTK_CONTAINER (vboxrecur), border);
 
   /* Button box */
@@ -794,52 +841,12 @@ build_edit_event_window (void)
 
   g_signal_connect (G_OBJECT (buttonok), "clicked",
                     G_CALLBACK (click_ok), window);
+  g_signal_connect (G_OBJECT (buttondelete), "clicked",
+                    G_CALLBACK (click_delete), window);
   g_signal_connect_swapped (G_OBJECT (buttoncancel), "clicked",
                             G_CALLBACK (edit_finished), window);
 
-  /* Alarm page */
-  alarmhbox1          = gtk_hbox_new (FALSE, boxspacing);
-  alarmhbox2          = gtk_hbox_new (FALSE, boxspacing);
-  scrolledwindowalarm = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindowalarm),
-                                         vboxalarm);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindowalarm),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-  alarmmenu           = gtk_menu_new ();
-  alarmbutton         = gtk_check_button_new_with_label (_("Alarm"));
-  alarmadj            = gtk_adjustment_new (5.0, 1.0, 100.0, 1.0, 5.0, 5.0);
-  alarmspin           = gtk_spin_button_new (GTK_ADJUSTMENT (alarmadj), 1.0, 0);
-  alarmoption         = gtk_option_menu_new ();
-  alarmlabel          = gtk_label_new (_("before event"));
-
-  gtk_menu_append (GTK_MENU (alarmmenu),
-                   gtk_menu_item_new_with_label (_("minutes")));
-  gtk_menu_append (GTK_MENU (alarmmenu),
-                   gtk_menu_item_new_with_label (_("hours")));
-  gtk_menu_append (GTK_MENU (alarmmenu),
-                   gtk_menu_item_new_with_label (_("days")));
-  gtk_menu_append (GTK_MENU (alarmmenu),
-                   gtk_menu_item_new_with_label (_("weeks")));
-  gtk_menu_append (GTK_MENU (alarmmenu),
-                   gtk_menu_item_new_with_label (_("months")));
-  gtk_menu_append (GTK_MENU (alarmmenu),
-                   gtk_menu_item_new_with_label (_("years")));
-
-  gtk_option_menu_set_menu (GTK_OPTION_MENU (alarmoption), alarmmenu);
-
-  g_signal_connect (G_OBJECT (alarmbutton), "clicked",
-                    G_CALLBACK (recalculate_sensitivities), window);
-
-  gtk_box_pack_start (GTK_BOX (alarmhbox1), alarmbutton, FALSE, FALSE, 0);
-
-  gtk_box_pack_start (GTK_BOX (alarmhbox2), alarmspin, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (alarmhbox2), alarmoption, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (alarmhbox2), alarmlabel, FALSE, FALSE, 0);
-
-  gtk_box_pack_start (GTK_BOX (vboxalarm), alarmhbox1, FALSE, FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vboxalarm), alarmhbox2, FALSE, FALSE, 0);
-
-/* Begin repeat vbox */
+  /* Reccurence page */
 
   vboxrepeattop = gtk_vbox_new (FALSE, 0);
 
@@ -851,7 +858,7 @@ build_edit_event_window (void)
   vboxrecur = gtk_vbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (vboxrepeat), vboxrecur, FALSE, FALSE, 0);
 
-/* recurrence radio buttons */
+  /* recurrence radio buttons */
   recurtable = gtk_table_new (3, 2, FALSE);
   gtk_box_pack_start (GTK_BOX (vboxrecur), recurtable, FALSE, FALSE, 0);
 
@@ -1091,13 +1098,6 @@ build_edit_event_window (void)
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radiobuttonforever), TRUE);
 
   gtk_box_pack_start (GTK_BOX (vboxtop), notebookedit, TRUE, TRUE, 2);
-
-  scrolledwindowevent = gtk_scrolled_window_new (NULL, NULL);
-  gtk_scrolled_window_add_with_viewport (GTK_SCROLLED_WINDOW (scrolledwindowevent),
-                                         vboxevent);
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindowevent),
-                                  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
-
   gtk_notebook_append_page (GTK_NOTEBOOK (notebookedit), scrolledwindowevent,
                             labeleventpage);
   gtk_notebook_append_page (GTK_NOTEBOOK (notebookedit), scrolledwindowalarm,
@@ -1195,6 +1195,8 @@ new_event (time_t t, guint timesel)
       gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (s->starttime)->entry), buf);
       gtk_date_combo_set_date (GTK_DATE_COMBO (s->startdate),
                                tm.tm_year + 1900, tm.tm_mon, tm.tm_mday);
+      gtk_date_combo_set_date (GTK_DATE_COMBO (s->reminderdate),
+                               tm.tm_year + 1900, tm.tm_mon, tm.tm_mday);
       t += 60 * 60;
       localtime_r (&t, &tm);
       strftime (buf, sizeof(buf), TIMEFMT, &tm);
@@ -1229,8 +1231,6 @@ edit_event (event_t ev)
       gtk_window_set_title (GTK_WINDOW (w), _("Calendar: Edit event"));
 
       gtk_widget_set_sensitive (s->deletebutton, TRUE);
-      g_signal_connect (G_OBJECT (s->deletebutton), "clicked",
-                        G_CALLBACK (click_delete), ev);
       evd = event_db_get_details (ev);
       gtk_text_buffer_set_text (gtk_text_view_get_buffer (GTK_TEXT_VIEW (s->description)),
                                 evd->description, -1);
@@ -1242,6 +1242,8 @@ edit_event (event_t ev)
       gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (s->starttime)->entry), buf);
       gtk_date_combo_set_date (GTK_DATE_COMBO (s->startdate),
                                tm.tm_year + 1900, tm.tm_mon, tm.tm_mday);
+      gtk_date_combo_set_date (GTK_DATE_COMBO (s->reminderdate),
+                               tm.tm_year + 1900, tm.tm_mon, tm.tm_mday);
 
       end=ev->start+ev->duration;
       localtime_r (&end, &tm);
@@ -1258,8 +1260,23 @@ edit_event (event_t ev)
     
       if (ev->flags & FLAG_ALARM)
         {
+	  unsigned int unit = 0;	/* minutes */
+
           gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (s->alarmbutton), TRUE);
-          gtk_spin_button_set_value (GTK_SPIN_BUTTON (s->alarmspin), ev->alarm);
+
+	  if (ev->alarm)
+	    {
+	      unsigned int i;
+
+	      for (i = 0; i < 4; i++)
+		{
+		  if ((ev->alarm % alarm_multipliers[i]) == 0)
+		    unit = i;
+		}
+	    }
+
+          gtk_spin_button_set_value (GTK_SPIN_BUTTON (s->alarmspin), ev->alarm / alarm_multipliers[unit]);
+	  gtk_option_menu_set_history (GTK_OPTION_MENU (s->alarmoption), unit);
         }
       else
         gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (s->alarmbutton), FALSE);
@@ -1277,6 +1294,7 @@ edit_event (event_t ev)
             {
             case RECUR_NONE:
               abort ();
+
             case RECUR_DAILY:
               gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (s->radiobuttondaily), TRUE);
               gtk_spin_button_set_value (GTK_SPIN_BUTTON (s->dailyspin), r->increment);
@@ -1323,8 +1341,10 @@ edit_event (event_t ev)
                                             TRUE);
             }
         }
-
-        s->ev = ev;
+	else 
+	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (s->radiobuttonnone), TRUE);
+	
+	s->ev = ev;
     }
 
   return w;
