@@ -25,6 +25,7 @@
 
 #include "gtkdatesel.h"
 
+static GSList *day_events[32];
 static GtkWidget *datesel, *draw;
 static guint xp, xs, ys;
 static struct render_ctl *c_old;
@@ -84,14 +85,15 @@ button_press (GtkWidget *widget,
       else if (event->type == GDK_2BUTTON_PRESS)
 	{
 	  struct tm tm;
+	  time_t selected_time;
 	  localtime_r (&viewtime, &tm);
 	  tm.tm_year = c->popup.year - 1900;
 	  tm.tm_mon = c->popup.month;
 	  tm.tm_mday = c->popup.day;
-	  viewtime = mktime (&tm);
+	  selected_time = mktime (&tm);
 	  if (pop_window) 
 	    gtk_widget_destroy (pop_window);
-	  set_day_view ();    
+	  set_time_and_day_view (selected_time);    
 	}
     }
   
@@ -209,6 +211,7 @@ draw_expose_event (GtkWidget *widget,
 
       for (j = 0; j < (TOTAL_DAYS / 7); j++)
 	{
+	  gchar *buffer;
 	  guint d = i + (7 * j);
 	  struct render_ctl *c = &rc[d];
 	  guint y = (j + 1) * ys;
@@ -226,7 +229,7 @@ draw_expose_event (GtkWidget *widget,
 				  x, y, xs, ys);
 
 	      snprintf (buf, sizeof (buf), "%d", c->popup.day);
-	      gchar *buffer = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
+	      buffer = g_locale_to_utf8 (buf, -1, NULL, NULL, NULL);
 	      pango_layout_set_text (pl, buffer, strlen (buffer));
 	      pango_layout_get_pixel_extents (pl, &pr, NULL);
 	      w = pr.width;
@@ -310,7 +313,6 @@ month_view_update ()
   guint day;
   time_t start, end;
   struct tm tm_start, tm_end;
-  GSList *day_events[32];
   GSList *iter;
   guint days;
   guint year, month;
@@ -331,10 +333,8 @@ month_view_update ()
     {
       struct render_ctl *c = &rc[day];
       if (c->popup.events)
-	{
-	  event_db_list_destroy (c->popup.events);
-	  c->popup.events = NULL;
-	}
+	c->popup.events = NULL;
+
     }
 
   for (day = 1; day <= days; day++)
@@ -358,6 +358,7 @@ month_view_update ()
 	      end+=60*60;
       }
 
+      if (day_events[day]) event_db_list_destroy (day_events[day]);
       day_events[day] = event_db_list_for_period (start, end);
 
       for (iter = day_events[day]; iter; iter = iter->next)
